#include "AplVarList.h"

#include "HQConfIo.h"

AplVarList::AplVarList(QObject* qoParent, const char* cpName) : QObject(qoParent, cpName)
{
	qstDefaultHostName		= QString::fromUtf8("gpsntp.miz.nao.ac.jp (国立天文台)\nns.riken.go.jp (理化学研究所)\ngps.kek.jp (高ｴﾈﾙｷﾞｰ物理学研究所)\nntp.hokudai.ac.jp (北海道大)\nntp.tohoku.ac.jp (東北大)\neric.nc.u-tokyo.ac.jp (東京大)\nns.hiroshima-u.ac.jp (広島大)\nclock.nc.fukuoka-u.ac.jp (福岡大)\ntime-nw.nist.gov (米国標準技術局)\nntp.nasa.gov (NASA)\nTick.UH.EDU (MIT)");
	iDefaultHostCnt			= 11;

	iDefaultTimeoutMsec		= 1000;


	qstHostListStr			= qstDefaultHostName;
	iHostListCnt			= iDefaultHostCnt;
	iSelectedHost			= 0;

	iTimeoutMsec			= iDefaultTimeoutMsec;

	blUpNotfy2Adjust		= true;

	blAppAutoQuit			= false;

	blPPPstop				= false;
	blPPPstop2				= false;
	blAutoStart				= false;

	blSetSystemClock		= false;

	blIsAlreadyConnection	= false;
}

void AplVarList::setVarList(QString &qstHostList,int iSelected,int iTimeout,bool blNotfyAdj, bool blStop,bool blStop2,bool blStart,bool blAutoQuit)
{
	QString		qstListTmp("");
	QString		qstTmp("");

	int			iListCnt = 0;
	int			iListStrPt = 0;

	while(1)
	{
		iListStrPt = findNextHost(qstHostList,iListStrPt,qstTmp);

		if(qstTmp.length() > 0)
		{
			qstListTmp += (qstTmp + QString("\n"));
			iListCnt++;
		}

		if(iListStrPt == 0) break;
	}

	if(iListCnt == 0)
	{
		qstHostListStr	= qstDefaultHostName;
		iHostListCnt	= iDefaultHostCnt;
	}
	else
	{
		qstHostListStr	= qstListTmp;
		iHostListCnt	= iListCnt;
	}

	if(iSelected < 0)
	{
		iSelectedHost = 0;
	}
	else if(iSelected >= iHostListCnt)
	{
		iSelectedHost = iHostListCnt - 1;
	}
	else
	{
		iSelectedHost = iSelected;
	}

	if(iTimeout < 1)
	{
		iTimeoutMsec = 1;
	}
	else if(iTimeout > 9999)
	{
		iTimeoutMsec = 9999;
	}
	else
	{
		iTimeoutMsec = iTimeout;
	}

	blUpNotfy2Adjust = blNotfyAdj;
	blPPPstop = blStop;
	blPPPstop2 = blStop2;
	blAutoStart = blStart;
	blAppAutoQuit = blAutoQuit;
}

int AplVarList::getDefaultHostList(QString &qstHostList)
{
	qstHostList = qstDefaultHostName;

	return (int)qstHostList.length();
}

int AplVarList::getHostName(int iIndex,QString &qstHostName)
{
	int			iIndexPt = 0;
	int			iTmp;

	if(iIndex >= iHostListCnt) return -1;

	for(iTmp = 0;iTmp < (iIndex + 1);iTmp++)
	{
		iIndexPt = findNextHost(qstHostListStr,iIndexPt,qstHostName);
		if(iIndexPt == 0) break;
	}

	return (int)qstHostName.length();
}

int AplVarList::findNextHost(QString &qstHostList,int iIndex,QString &qstHostName)
{
	QString		qstTmp;
	int			iResult;
	int			iTmp;

	iTmp = qstHostList.find((char)0x0a,iIndex,TRUE);

	switch(iTmp)
	{
		case 0:
			qstHostName = "";
			iResult = iIndex + 1;
			break;

		case -1:
			qstHostName = qstHostList.mid((uint)iIndex);
			iResult = 0;
			break;

		default:
			qstHostName = qstHostList.mid((uint)iIndex,(uint)(iTmp - iIndex));
			iResult = iTmp + 1;
			break;
	}

	qstHostName = qstHostName.stripWhiteSpace();

	return iResult;
}

bool AplVarList::loadVarList()
{
#ifdef DTK_APPLET
	QString				qstAppname("dtkapplet");
#else
	QString				qstAppname("dtktime");
#endif

	QString				qstGroup;
	QString				qstTag;

	int					iFuncResult = true;

	HQConfIo*			hqConfRead;

	QString				qstTmp;
	int					iTmp;
	int					iCnt;
	int					iVal;

	/* init */
	qstHostListStr		= "";
	iHostListCnt		= 0;
	iTimeoutMsec		= iDefaultTimeoutMsec;

	blUpNotfy2Adjust	= true;
	blPPPstop			= false;
	blPPPstop2			= false;
	blAutoStart			= false;
	blAppAutoQuit		= false;

	hqConfRead = new HQConfIo( qstAppname );

	hqConfRead->setCodecName("Shift-JIS");

	iTmp = hqConfRead->setLoadMode();
	if(iTmp == 1)
	{
		/* Host */
		qstGroup = "NtpHost";
		for(iCnt = 1;iCnt < 10001;iCnt++)
		{
			qstTag = "Host" + QString::number(iCnt);
			if( hqConfRead->getStrValue(qstGroup, qstTag		, qstTmp			, ""				, 1000		) == true )
			{
				qstTmp = qstTmp.stripWhiteSpace();
				if(qstTmp.length() > 0)
				{
					qstHostListStr += (qstTmp + "\n");
					iHostListCnt++;
				}
			}
			else
			{
				break;
			}
		}

		if( hqConfRead->getIntValue(qstGroup, "Selected"		, iSelectedHost		, 0					, 0		, 10000	) == false ) iFuncResult = false;

		/* TimeOut */
		qstGroup = "Timeout";

		if( hqConfRead->getIntValue(qstGroup, "Timeout"			, iTimeoutMsec		, iTimeoutMsec		, 1		, 9999	) == false ) iFuncResult = false;


		/* Setting */
		qstGroup = "Setting";

		if( hqConfRead->getIntValue(qstGroup, "Notyf2Adjust"	, iVal				, blUpNotfy2Adjust	, 0		, 1		) == false ) iFuncResult = false;
		blUpNotfy2Adjust= iVal;

#ifndef DTK_APPLET
		if( hqConfRead->getIntValue(qstGroup, "DisConnect"		, iVal				, blPPPstop			, 0		, 1		) == false ) iFuncResult = false;
		blPPPstop = iVal;

		if( hqConfRead->getIntValue(qstGroup, "DisConnect2"		, iVal				, blPPPstop2		, 0		, 1		) == false ) iFuncResult = false;
		blPPPstop2 = iVal;
#endif

		if( hqConfRead->getIntValue(qstGroup, "AutoStart"		, iVal				, blAutoStart		, 0		, 1		) == false ) iFuncResult = false;
		blAutoStart = iVal;

		if( hqConfRead->getIntValue(qstGroup, "AutoQuit"		, iVal				, blAppAutoQuit		, 0		, 1		) == false ) iFuncResult = false;
		blAppAutoQuit = iVal;
	}
	else if( iTmp == 0 )
	{
		/* confファイルが無い */
	}
	else
	{
		// -1: ファイル	I/Oエラーが発生した
		iFuncResult = false;
	}

	if(iHostListCnt == 0)
	{
		qstHostListStr = qstDefaultHostName;
		iHostListCnt = iDefaultHostCnt;
	}

	if(iSelectedHost >= iHostListCnt)
	{
		iSelectedHost = iHostListCnt - 1;
	}

	delete hqConfRead;

	return iFuncResult;
}

bool AplVarList::saveVarList()
{
#ifdef DTK_APPLET
	QString			qstAppname("dtkapplet");
#else
	QString			qstAppname("dtktime");
#endif

	QString			qstGroup;
	QString			qstTag;

	QString			qstTmp;
	int				iCnt;
	int				iIndexPt;

	bool			blNoError = true;

	HQConfIo		*hqConfWrite;

	hqConfWrite = new HQConfIo( qstAppname );

	hqConfWrite->setCodecName("Shift-JIS");

	if(hqConfWrite->setSaveMode() == true)
	{
		/* Host */
		qstGroup = "NtpHost";
		if( hqConfWrite->setGroup(qstGroup) == false) blNoError = false;

		iIndexPt	= 0;
		iCnt		= 0;
		while(1)
		{
			iIndexPt = findNextHost(qstHostListStr,iIndexPt,qstTmp);

			if(qstTmp.length() > 0)
			{
				iCnt++;

				qstTag = "Host" + QString::number(iCnt);
				hqConfWrite->setStrValue( qstTag		, qstTmp		);
			}

			if(iIndexPt == 0) break;
		}

		hqConfWrite->setIntValue( "Selected"		, iSelectedHost			);

		/* TimeOut */
		qstGroup = "Timeout";
		hqConfWrite->setGroup(qstGroup);
		hqConfWrite->setIntValue( "Timeout"			, iTimeoutMsec			);

		/* Setting */
		qstGroup = "Setting";
		hqConfWrite->setGroup(qstGroup);
		hqConfWrite->setIntValue( "Notyf2Adjust"	, (int)blUpNotfy2Adjust	);

#ifndef DTK_APPLET
		hqConfWrite->setIntValue( "DisConnect"		, (int)blPPPstop		);
		hqConfWrite->setIntValue( "DisConnect2"		, (int)blPPPstop2		);
#endif

		hqConfWrite->setIntValue( "AutoStart"		, (int)blAutoStart		);
		hqConfWrite->setIntValue( "AutoQuit"		, (int)blAppAutoQuit	);
	}
	else
	{
		blNoError = false;
	}

	if(blNoError == true) hqConfWrite->flash();

	delete hqConfWrite;

	return blNoError;
}

bool AplVarList::writeTrigger()
{
	QFile		qflFile(QString("/var/spool/at/trigger"));

	if(qflFile.open(IO_WriteOnly) == false)
	{
		return false;
	}
	else
	{
		QTextStream		qtsStream( &qflFile );

		qtsStream << QString("W");
		qtsStream << endl;

		qflFile.close();
	}

	return true;
}
