#include "HQConfIo.h"

HQConfIo::HQConfIo(const QString& qstAppName)
{
	iIO_Mode		= 0;
	iMaxVarCnt		= 0;

	qstCodecName	= "";

	if( qstAppName == "" )
	{
		qflFileHandle = new QFile();
	}
	else
	{
		QString		qstFilename = QDir::homeDirPath() + "/Settings/" + qstAppName + ".conf";

		qflFileHandle = new QFile(qstFilename);
	}
}

HQConfIo::~HQConfIo()
{
	if( iIO_Mode > 0 ) qflFileHandle->close();

	delete qflFileHandle;
}

void HQConfIo::setFileName(const QString& qstFilename)
{
	qflFileHandle->setName(qstFilename);
}

void HQConfIo::setCodecName(const QString& qstCodec)
{
	qstCodecName = qstCodec;
}

// save系関数
bool HQConfIo::setSaveMode()
{
	if( iIO_Mode == 0 )
	{
		if(qflFileHandle->open(IO_WriteOnly) == false)
		{
			iIO_Mode = 0;

			return false;
		}
		else
		{
			iIO_Mode = 2;	/* Save Moded */

			return true;
		}
	}
	else
	{
		return false;
	}
}

bool HQConfIo::setGroup(const QString &qstGroup)
{
	if( iIO_Mode != 2 ) return false;		/* Mode check */

	QString		qstTmp = "[" + qstGroup.stripWhiteSpace() + "]";

	qstVarBuf.append( qstTmp );

	iMaxVarCnt++;

	return true;
}

bool HQConfIo::setStrValue(const QString &qstTag,const QString &qstValue)
{
	if( iIO_Mode != 2 ) return false;		/* Mode check */

	QString		qstTmp = qstTag + "=" + qstValue;

	qstVarBuf.append( qstTmp );

	iMaxVarCnt++;

	return true;
}

bool HQConfIo::setIntValue(const QString &qstTag, int iValue)
{
	if( iIO_Mode != 2 ) return false;		/* Mode check */

	QString		qstTmp;

	qstTmp.sprintf("=%d",iValue);
	qstTmp = qstTag + qstTmp;

	qstVarBuf.append( qstTmp );

	iMaxVarCnt++;

	return true;
}

bool HQConfIo::flash()
{
	if( iIO_Mode != 2 ) return false;		/* Mode check */

	if( iMaxVarCnt > 0 )
	{
		int		iCnt;

		QTextStream		qtsStream( qflFileHandle );

		if(qstCodecName != "")
		{
			qtsStream.setCodec( QTextCodec::codecForName(qstCodecName.latin1()) );
		}

		for(iCnt = 0;iCnt < iMaxVarCnt;iCnt++)
		{
			qtsStream << qstVarBuf[iCnt];
			qtsStream << endl;
		}
	}

	return true;
}

// load系関数
int HQConfIo::setLoadMode()
{
	if( iIO_Mode == 0 )
	{
		if(qflFileHandle->open(IO_ReadOnly) == false)
		{
			iIO_Mode = 0;

			return 0;
		}
		else
		{
			iIO_Mode = 1;	/* Load Mode */

			if(readConfFile() == true)
			{
				return 1;
			}
			else
			{
				return -1;
			}
		}
	}
	else
	{
		return 0;
	}
}

bool HQConfIo::getStrValue(const QString& qstGroup,const QString& qstTag,QString& qstResultValue,QString qstDefaultValue,int iMaxLen)
{
	if( iIO_Mode != 1 ) return false;		/* Mode check */

	QString			qstTmp;

	if( searchValue(qstGroup,qstTag,qstTmp) == true )
	{
		if( (iMaxLen > 0) && ((int)qstTmp.length() > iMaxLen) )
		{
			qstTmp = qstTmp.left(iMaxLen);
		}
		qstResultValue = qstTmp;
		return true;
	}

	qstResultValue = qstDefaultValue;
	return false;
}

bool HQConfIo::getIntValue(const QString& qstGroup,const QString& qstTag,int& iResultValue,int iDefaultValue,int iMinLimit,int iMaxLimit)
{
	if( iIO_Mode != 1 ) return false;		/* Mode check */

	QString			qstTmp;
	int				iTmp;

	if( searchValue(qstGroup,qstTag,qstTmp) == true )
	{
		bool	blOk;

		qstTmp = qstTmp.stripWhiteSpace();
		iTmp = qstTmp.toInt(&blOk);

		if(blOk == true)
		{
			if( !((iMinLimit == 0) && (iMaxLimit == 0)) )
			{
				if( (iTmp >= iMinLimit) && (iTmp <= iMaxLimit) )
				{
					iResultValue = iTmp;
					return true;
				}
			}
		}
	}

	iResultValue = iDefaultValue;
	return false;
}

bool HQConfIo::searchValue(const QString& qstGroup,const QString& qstTag,QString& qstValue)
{
	// private 関数
	int			iCnt1,iCnt2,iP1;
	QString		qstLineBuf;

	for(iCnt1 = 0;iCnt1 < iMaxVarCnt;iCnt1++)
	{
		qstLineBuf = qstVarBuf[iCnt1];

		if( qstLineBuf.left(1) == "[" )
		{
			qstLineBuf.remove(0,1);

			if( qstLineBuf == qstGroup )
			{
				for(iCnt2 = iCnt1 + 1;iCnt2 < iMaxVarCnt;iCnt2++)
				{
					qstLineBuf = qstVarBuf[iCnt2];

					if( qstLineBuf.left(1) == "[" )
					{
						iCnt2--;
						break;
					}

					iP1 = qstLineBuf.find("=");
					if(iP1 > 0)
					{
						if( qstLineBuf.left(iP1) == qstTag )
						{
							qstLineBuf.remove(0, iP1 + 1);

							qstValue = qstLineBuf;
							return true;
						}
					}
				}

				iCnt1 = iCnt2;
			}
		}
	}

	return false;
}

bool HQConfIo::readConfFile()
{
	// private 関数
	int				iP1,iP2;
	QString			qstTmp1;
	QString			qstTmp2;

	QTextStream		qtsStream( qflFileHandle );

	if(qstCodecName != "")
	{
		qtsStream.setCodec( QTextCodec::codecForName(qstCodecName.latin1()) );
	}

	while( qtsStream.atEnd() == false )
	{
		qstTmp1 = qstTmp2 = qtsStream.readLine();

		qstTmp1 = qstTmp1.stripWhiteSpace();
		qstTmp2 = stripLeftSpace( qstTmp2 );

		iP1 = qstTmp1.find("]",1);
		iP2 = qstTmp2.find("=",1);
		if( (qstTmp1.find("[") == 0) && (iP1 != -1) )
		{
			qstTmp1 = qstTmp1.left(iP1);
			qstVarBuf.append( qstTmp1 );
			iMaxVarCnt++;
		}
		else if(iP2 != -1)
		{
			qstTmp1 = qstTmp2.left(iP2);
			qstTmp1 = qstTmp1.stripWhiteSpace() + "=";
			qstTmp2.remove(0,iP2 + 1);
			qstTmp2 = stripLeftSpace( qstTmp2 );

			qstTmp1 += qstTmp2;
			qstVarBuf.append( qstTmp1 );

			iMaxVarCnt++;
		}
	}

	return true;
}

QString HQConfIo::stripLeftSpace(QString qstString)
{
	// private 関数
	while( qstString[0].isSpace() )
	{
		qstString.remove(0,1);
	}

	return qstString;
}
