#include "HQSntp.h"

HQSntp::HQSntp(const QString& qstHostname, int iConectport, int iTimeout, bool blAutoQuit, QObject* qoParent, const char* cpName) : QObject(qoParent, cpName)
{
	qwdParent			= (QWidget*)qoParent;

	qscSocketDev		= new QSocketDevice(QSocketDevice::Datagram);
	qscSocketDev->setBlocking(FALSE);
//	qscSocketDev->setReceiveBufferSize(256);
//	qscSocketDev->setReceiveBufferSize(1024);

	qstConnectHostStr	= qstHostname;
	iConnectHostPort	= iConectport;

	qhaInAddrAny.setAddress((Q_UINT32)0);	// 受信用アドレス:0.0.0.0 自動設定
	iLocalPort			= 0;	// 受信ポート:0 自動設定

	iTimeOutMsec		= iTimeout;

	blAutoQuitNotify	= blAutoQuit;

	qsnReadNotify		= new QSocketNotifier( qscSocketDev->socket(), QSocketNotifier::Read );
	qsnWriteNotify		= new QSocketNotifier( qscSocketDev->socket(), QSocketNotifier::Write );

	qtrTimeoutNotify	= new QTimer( qwdParent );	// タイムアウト監視用タイマ

	qdnDns				= NULL;

	connect( qsnReadNotify, SIGNAL(activated(int)), this, SLOT(packetReadNotifySlot()) );
	qsnReadNotify->setEnabled(FALSE);

	connect( qsnWriteNotify, SIGNAL(activated(int)), this, SLOT(packetWriteNotifySlot()) );
	qsnWriteNotify->setEnabled(FALSE);

	connect( qtrTimeoutNotify, SIGNAL(timeout()), this, SLOT(timeoutNotfySlot()) );
}

HQSntp::~HQSntp()
{
	if(qdnDns != NULL)
	{
		disconnect(qdnDns);
		delete qdnDns;
	}

	qtrTimeoutNotify->stop();
	qsnReadNotify->setEnabled(false);
	qsnWriteNotify->setEnabled(false);

	disconnect(qsnReadNotify);
	disconnect(qsnWriteNotify);
	disconnect(qtrTimeoutNotify);

	delete qsnReadNotify;
	delete qsnWriteNotify;
	delete qtrTimeoutNotify;
	delete qscSocketDev;
}

void HQSntp::sendPacketExecSlot()
{
	QValueList<QHostAddress>	qvlHostList;

	QString		qstMsgStr;

	qtrTimeoutNotify->stop();	// タイムアウト監視タイマ：停止

	qvlHostList = qdnDns->addresses();
	if(qvlHostList.isEmpty() == true)
	{
		// empty
		qstMsgStr = QString::fromUtf8("NTPｻｰﾊﾞｰが見つかりませんでした\n");
		requestDispMessage(qstMsgStr);

		requestDisposeObj();
		return;
	}

	qhaConnectHostAddr = qvlHostList[0];

	qstMsgStr = QString::fromUtf8("NTPｻｰﾊﾞｰを確認:");
	qstMsgStr += qstConnectHostStr;
	requestDispMessage(qstMsgStr);

	qstMsgStr = QString::fromUtf8("IP Address:");
	qstMsgStr += qhaConnectHostAddr.toString() + "\n";
	requestDispMessage(qstMsgStr);

	qtrTimeoutNotify->start( iTimeOutMsec, TRUE );	// タイムアウト監視タイマ起動

	qsnWriteNotify->setEnabled(TRUE);	// Packet書込み許可の通知を開始
}

void HQSntp::packetWriteNotifySlot()
{
	QString		qstMsgStr;

	timeval		tvTransmitTs;
	timezone	tzTimeZone;
	int			iResult;

	qsnWriteNotify->setEnabled(FALSE);
	qsnReadNotify->setEnabled(TRUE);

	iResult = gettimeofday(&tvTransmitTs,&tzTimeZone);
	if(iResult != 0)
	{
		qstMsgStr = QString::fromUtf8("ｼｽﾃﾑ時刻の取得に失敗しました\n");

		requestDispMessage(qstMsgStr);
		requestDisposeObj();
		return;
	}

	setBuf4Timeval(hspSendPacket,&tvTransmitTs,40);	// Transmit Time Stamp に現在のシステム時刻をSET

	iResult = qscSocketDev->writeBlock(hspSendPacket,48,qhaConnectHostAddr,iConnectHostPort);
	if(iResult != 48)
	{
		qsnReadNotify->setEnabled(FALSE);

		qstMsgStr = QString::fromUtf8("ﾊﾟｹｯﾄの送信に失敗しました\n");
		requestDispMessage(qstMsgStr);
		requestDisposeObj();
	}
	else
	{
		qstMsgStr = QString::fromUtf8("NTPｻｰﾊﾞｰにｺﾏﾝﾄﾞを送信しています\n");
		requestDispMessage(qstMsgStr);
	}
}

void HQSntp::packetReadNotifySlot()
{
	int			iResult;

	qsnReadNotify->setEnabled(FALSE);

	qtrTimeoutNotify->stop();

	iResult = qscSocketDev->readBlock(hspRecvPacket,48);
	if(iResult != 48)
	{
		QString		qstMsgStr;

		qstMsgStr = QString::fromUtf8("ﾊﾟｹｯﾄの受信に失敗しました\n");

		requestDispMessage(qstMsgStr);
		requestDisposeObj();
		return;
	}

	adjustSystemTime();
}

void HQSntp::adjustSystemTime()
{
	timeval		tvReferenceTs;

	timeval		tvOriginateTs;
	timeval		tvReceiveTs;
	timeval		tvTransmitTs;
	timeval		tvDestinationTs;

	timeval		tvD;
	timeval		tvT;

	timeval		tvDispT;

	timeval		tvTmp1,tvTmp2;

	timezone	tzTimeZone;

	int			iResult;

	QString		qstMsgStr;

	QString		qstTmp;

	// タイムサーバの情報を取得・表示
	iResult = getIntVal4Buf(hspRecvPacket,0);
	{
		int		iLI,iVN,iMode,iStratum,iPoll,iPrecision;

		iLI			= (iResult & 0xc0000000) >> 30;
		iVN			= (iResult & 0x38000000) >> 27;
		iMode		= (iResult & 0x07000000) >> 24;
		iStratum	= (iResult & 0x00ff0000) >> 16;
		iPoll		= (iResult & 0x0000ff00) >> 8;
		iPrecision	= (char)(iResult & 0x000000ff);

		if(iPrecision > 127) iPrecision -= 0x100;

		qstMsgStr.sprintf(">LI:%d VN:%d Mode:%d Stratum:%d Poll:%d Precision:%d\n",iLI ,iVN ,iMode ,iStratum ,iPoll ,iPrecision);
		requestDispMessage(qstMsgStr);
	}

	// システム時刻を DestinationTimeStamp にセット
	iResult = gettimeofday(&tvDestinationTs,&tzTimeZone);
	if(iResult != 0)
	{
		qstMsgStr = QString::fromUtf8("ｼｽﾃﾑ時刻の取得に失敗しました\n");

		requestDispMessage(qstMsgStr);
		requestDisposeObj();
	}

	// NTPサーバからの返答をセット
	getTimeval4buf(&tvReferenceTs,hspRecvPacket,16);
	getTimeval4buf(&tvOriginateTs,hspRecvPacket,24);		/* T1 */
	getTimeval4buf(&tvReceiveTs,hspRecvPacket,32);			/* T2 */
	getTimeval4buf(&tvTransmitTs,hspRecvPacket,40);			/* T3 */
															/* T4 = tvDestinationTs */

	// ルート遅延を算出
	timevalSub(&tvTmp1,&tvDestinationTs,&tvOriginateTs);	/* (T4 - T1):Ta */
	timevalSub(&tvTmp2,&tvReceiveTs,&tvTransmitTs);			/* (T2 - T3):Tb */
	timevalSub(&tvD,&tvTmp1,&tvTmp2);						/* (Ta - Tb):d */

	// システム時刻との時差を算出
	timevalSub(&tvTmp1,&tvReceiveTs,&tvOriginateTs);		/* (T2 - T1):Ta */
	timevalSub(&tvTmp2,&tvTransmitTs,&tvDestinationTs);		/* (T3 - T4):Tb */
	timevalAdd(&tvTmp1,&tvTmp2);							/* (Ta + Tb):Tc */
	timevalDiv(&tvT,&tvTmp1,2);								/* (Tc / 2) :t */

	// 画面表示用の時差情報を算出
	if( (tvT.tv_sec < 0) && (tvT.tv_usec > 0) )
	{
		tvDispT.tv_sec = tvT.tv_sec + 1;
		tvDispT.tv_usec = 1000000 - tvT.tv_usec;
	}
	else
	{
		tvDispT.tv_sec = tvT.tv_sec;
		tvDispT.tv_usec = tvT.tv_usec;
	}

#ifdef DMODE
		// debug only↓ 動作確認用
		QDateTime	qdtDateTime;
		QDate		qdtDate;
		QTime		qtmTime;

		qstMsgStr = "<NTP Server Response>";
		requestDispMessage(qstMsgStr);

		qdtDateTime.setTime_t((unsigned int)tvReferenceTs.tv_sec);
		qdtDate = qdtDateTime.date();
		qtmTime = qdtDateTime.time();
		qstMsgStr.sprintf("NTP SvrRefTs: %d/%d/%d %d:%d:%d.%.6d",qdtDate.year(), qdtDate.month(), qdtDate.day(),
													  qtmTime.hour(), qtmTime.minute(), qtmTime.second(),
													  (int)tvReferenceTs.tv_usec);
		requestDispMessage(qstMsgStr);

		qdtDateTime.setTime_t((unsigned int)tvOriginateTs.tv_sec);
		qdtDate = qdtDateTime.date();
		qtmTime = qdtDateTime.time();
		qstMsgStr.sprintf("OriginateTs : %d/%d/%d %d:%d:%d.%.6d",qdtDate.year(), qdtDate.month(), qdtDate.day(),
													  qtmTime.hour(), qtmTime.minute(), qtmTime.second(),
													  (int)tvOriginateTs.tv_usec);
		requestDispMessage(qstMsgStr);

		qdtDateTime.setTime_t((unsigned int)tvReceiveTs.tv_sec);
		qdtDate = qdtDateTime.date();
		qtmTime = qdtDateTime.time();
		qstMsgStr.sprintf("ReceiveTs   : %d/%d/%d %d:%d:%d.%.6d",qdtDate.year(), qdtDate.month(), qdtDate.day(),
													  qtmTime.hour(), qtmTime.minute(), qtmTime.second(),
													  (int)tvReceiveTs.tv_usec);
		requestDispMessage(qstMsgStr);

		qdtDateTime.setTime_t((unsigned int)tvTransmitTs.tv_sec);
		qdtDate = qdtDateTime.date();
		qtmTime = qdtDateTime.time();
		qstMsgStr.sprintf("TransmitTs  : %d/%d/%d %d:%d:%d.%.6d",qdtDate.year(), qdtDate.month(), qdtDate.day(),
													  qtmTime.hour(), qtmTime.minute(), qtmTime.second(),
													  (int)tvTransmitTs.tv_usec);
		requestDispMessage(qstMsgStr);

		qdtDateTime.setTime_t((unsigned int)tvDestinationTs.tv_sec);
		qdtDate = qdtDateTime.date();
		qtmTime = qdtDateTime.time();
		qstMsgStr.sprintf("Destination : %d/%d/%d %d:%d:%d.%.6d",qdtDate.year(), qdtDate.month(), qdtDate.day(),
													  qtmTime.hour(), qtmTime.minute(), qtmTime.second(),
													  (int)tvDestinationTs.tv_usec);
		requestDispMessage(qstMsgStr);

		qstMsgStr = "<Result>";
		requestDispMessage(qstMsgStr);

		qdtDateTime.setTime_t((unsigned int)tvD.tv_sec);
		qdtDate = qdtDateTime.date();
		qtmTime = qdtDateTime.time();
		qstMsgStr.sprintf("Root Delay  : %d.%.6d sec", qtmTime.second(), (int)tvD.tv_usec);
		requestDispMessage(qstMsgStr);

		qstMsgStr.sprintf("Difference  : %d.%.6d sec",(int)tvT.tv_sec,(int)tvT.tv_usec);
		requestDispMessage(qstMsgStr);

		if(tvDispT.tv_sec < 0)
		{
			qstMsgStr.sprintf("DispDiff    : %d.%.6d sec delay",(int)(tvDispT.tv_sec * -1),(int)tvDispT.tv_usec);
		}
		else
		{
			qstMsgStr.sprintf("DispDiff    : %d.%.6d sec fase",(int)tvDispT.tv_sec,(int)tvDispT.tv_usec);
		}
		requestDispMessage(qstMsgStr);
		// debug only↑
#else
		qstMsgStr = QString::fromUtf8("反応に");
		qstTmp.sprintf(" %d.%.6d ", (int)tvD.tv_sec, (int)tvD.tv_usec);
		qstMsgStr += qstTmp + QString::fromUtf8("秒かかりました");

		requestDispMessage(qstMsgStr);

		if(tvDispT.tv_sec < 0)
		{
			qstTmp.sprintf("%d.%.6d ",(int)(tvDispT.tv_sec * -1),(int)tvDispT.tv_usec);
			qstMsgStr = qstTmp + QString::fromUtf8("秒進んでいます");
		}
		else
		{
			qstTmp.sprintf("%d.%.6d ",(int)tvDispT.tv_sec,(int)tvDispT.tv_usec);
			qstMsgStr = qstTmp + QString::fromUtf8("秒遅れています");
		}
		requestDispMessage(qstMsgStr);
#endif

	iResult = gettimeofday(&tvTmp1,&tzTimeZone);
	if(iResult != 0)
	{
		qstMsgStr = QString::fromUtf8("ｼｽﾃﾑ時刻の取得に失敗しました\n");

		requestDispMessage(qstMsgStr);
		requestDisposeObj();
	}

	// 時差を調整
	timevalAdd(&tvTmp1,&tvT);

	iResult = settimeofday(&tvTmp1,&tzTimeZone);
	if(iResult != 0)
	{
		qstMsgStr = QString::fromUtf8("ｼｽﾃﾑ時刻の調整に失敗しました");
		requestDispMessage(qstMsgStr);
	}
	else
	{
		qstMsgStr = QString::fromUtf8("ｼｽﾃﾑ時刻を調整しました\n");
		requestDispMessage(qstMsgStr);

		emit setSystemClockSignal();

		if(blAutoQuitNotify == true) emit requestAutoQuitRequestSignal();
	}

	requestDisposeObj();
}

void HQSntp::timeoutNotfySlot()
{
	QString		qstMsgStr;

	qstMsgStr = QString::fromUtf8("ﾀｲﾑｱｳﾄ\n");

	qtrTimeoutNotify->stop();

	requestDispMessage(qstMsgStr);
	requestDisposeObj();
}

void HQSntp::sendNtpPacket()
{
	QString		qstMsgStr;

	int			iControlWord;

	qstMsgStr = QString::fromUtf8("[NTPｻｰﾊﾞｰとの交信を開始]\n");
	requestDispMessage(qstMsgStr);

	memset(hspSendPacket,0x00,48);

	// コントロールコード
	iControlWord = htonl(0x1B000000);	// LI0 VN3 Mode3
	memcpy(hspSendPacket,(char *)&iControlWord,4);

	if(qscSocketDev->bind(qhaInAddrAny,iLocalPort) != TRUE)
	{
		qstMsgStr = QString::fromUtf8("ﾛｰｶﾙﾎﾟｰﾄとBINDできませんでした\n");
		requestDispMessage(qstMsgStr);

		requestDisposeObj();
		return;
	}
	else
	{
		qstMsgStr = QString::fromUtf8("ﾛｰｶﾙﾎﾟｰﾄとBINDしました");
		requestDispMessage(qstMsgStr);
	}

	qdnDns		= new QDns( qstConnectHostStr, QDns::A );
	connect( qdnDns, SIGNAL(resultsReady()), this, SLOT(sendPacketExecSlot()) );

	qtrTimeoutNotify->start( iTimeOutMsec, TRUE );	// タイムアウト監視を開始
}

void HQSntp::requestDisposeObj(void)
{
	emit requestDisposeHQSntpSignal();
}

void HQSntp::requestDispMessage(QString &qstDispStr)
{
	QString		qstEmitStr(qstDispStr);

	emit requestDispMessageHQSntpSignal(qstEmitStr);
}

int HQSntp::timevalSub(struct timeval* tvResult,struct timeval* tvA,struct timeval* tvB)
{
	tvResult->tv_sec = tvA->tv_sec - tvB->tv_sec;
	tvResult->tv_usec = tvA->tv_usec - tvB->tv_usec;

	if(tvResult->tv_usec < 0)
	{
		tvResult->tv_usec += 1000000;
		--tvResult->tv_sec;
	}

	if( (tvResult->tv_sec == 0) && (tvResult->tv_usec == 0) )
	{
		return 0;
	}
	else if( tvResult->tv_sec < 0 )
	{
		return -1;
	}
	else
	{
		return 1;
	}
}

void HQSntp::timevalAdd(struct timeval* tvA,struct timeval* tvB)
{
	tvA->tv_sec += tvB->tv_sec;
	tvA->tv_usec += tvB->tv_usec;

	if( tvA->tv_usec >= 1000000 )
	{
		tvA->tv_sec += tvA->tv_usec / 1000000;
		tvA->tv_usec %= 1000000;
	}
}

void HQSntp::timevalDiv(struct timeval* tvResult,struct timeval* tvA,int iDiv)
{
	float		fTmp;

	tvResult->tv_sec = tvA->tv_sec / iDiv;

	fTmp = ((float)(tvA->tv_sec % iDiv) / (float)iDiv) * (float)1000000;

	tvResult->tv_usec = tvA->tv_usec / iDiv;
	tvResult->tv_usec += (int)fTmp;

	if(tvResult->tv_usec < 0)
	{
		tvResult->tv_usec += 1000000;
		--tvResult->tv_sec;
	}
}

int HQSntp::getIntVal4Buf(char *cpBuf,int iOffset)
{
	int		iTmp;

	memcpy((char *)&iTmp,cpBuf + iOffset, 4);
	iTmp = ntohl(iTmp);

	return iTmp;
}

void HQSntp::getTimeval4buf(struct timeval* tvResult,char *cpBuf,int iOffset)
{
	int		i32Tmp;

	memcpy((char *)&i32Tmp,cpBuf + iOffset,4);
//	tvResult->tv_sec = ntohl(i32Tmp) - 2208988800;
	tvResult->tv_sec = ntohl(i32Tmp) - (uint)0x83AA7E80;

	memcpy((char *)&i32Tmp,cpBuf + iOffset + 4,4);
	tvResult->tv_usec = (uint)((float)ntohl(i32Tmp) / (float)4294.967296);	// fraction / (2^32 / 1000000) = usec
}

void HQSntp::setBuf4Timeval(char *cpBuf,struct timeval* tvVal,int iOffset)
{
	int		i32Tmp;

//	i32Tmp = htonl(tvVal->tv_sec + 2208988800);
	i32Tmp = htonl(tvVal->tv_sec + (uint)0x83AA7E80);
	memcpy(cpBuf + iOffset,(char *)&i32Tmp,4);

	i32Tmp = htonl( (uint)((float)tvVal->tv_usec * (float)4294.967296) );
	memcpy(cpBuf + iOffset + 4,(char *)&i32Tmp,4);
}
