#include <qpe/qpeapplication.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qspinbox.h>
#include <qcheckbox.h>

#include <sl/sldevinfo.h>

#include "MainFm.h"

static const char* const timeIcon_data[] = {
"16 16 5 1",
". c None",
"a c #000000",
"c c #800000",
"b c #c0c0c0",
"# c #ffffff",
"......#####.....",
"....##aaaaa.....",
"...#aabb#bbaa...",
"..#abbb#cbbb#a..",
".#abbbb#cbbbb#a.",
".#abbbb#cbbbb#a.",
"#abbbbb#cbbbbb#a",
"#abbbbb#c##bbb#a",
"#abbbbbbccccbb#a",
"#abbbbbbbbbbbb#a",
".a#bbbbbbbbbbb#a",
"..abbbbbbbbbb#a.",
"..a#bbbbbbbbb#a.",
"...a##bbbbb##a..",
"....aa#####aa...",
"......aaaaa....."};

static const char* const settingIcon_data[] = {
"16 16 15 1",
". c None",
"l c #000000",
"b c #212121",
"# c #424242",
"h c #636363",
"k c #6b6b6b",
"a c #7b7b7b",
"g c #848484",
"f c #949494",
"e c #9c9c9c",
"j c #adadad",
"d c #bdbdbd",
"i c #dedede",
"m c #efefef",
"c c #ffffff",
".......#aaaab...",
"......#cdeeef#..",
".....#cdgg###h..",
".....aigg#ggh.jh",
".....aig#gb..kg#",
".....aig#g...hg#",
".....aig#g..lfg#",
".....amgh#lleig#",
"....#cmiiiiiifg#",
"...#cmdfgggggg#h",
"..#cmdfg######h.",
".#cmdfgbhggggh..",
"#cmdfgbhb.......",
"cmdfgbhb........",
"mdfgbhb.........",
".fgbhb.........."};

MainFm::MainFm( QWidget* parent,  const char* name, WFlags fl ) : MainUI( parent, name, fl )
{
#ifdef DTK_APPLET
	appletCaptionLbl->setText( QString::fromUtf8( "どんとこいタイム Applet Ver0.9.0β" ) );
#else
	setCaption( QString::fromUtf8( "どんとこいタイム Ver1.1.0" ) );
#endif

	/* SettingPageタブ配置  */
	tabID2 = new QWidget( selectTab, "tabID2" );
	tabID2Layout = new QGridLayout( tabID2 );
	tabID2Layout->setSpacing( 3 );
	tabID2Layout->setMargin( 2 );

	button2Frm = new QFrame( tabID2, "button2Frm" );
	button2Frm->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, button2Frm->sizePolicy().hasHeightForWidth() ) );
	button2Frm->setFrameShape( QFrame::NoFrame );
	button2Frm->setFrameShadow( QFrame::Raised );
	button2FrmLayout = new QGridLayout( button2Frm );
	button2FrmLayout->setSpacing( 3 );
	button2FrmLayout->setMargin( 3 );
	QSpacerItem* spacer_3 = new QSpacerItem( 16, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	button2FrmLayout->addItem( spacer_3, 0, 2 );

	okSetBtn = new QPushButton( button2Frm, "okSetBtn" );
	okSetBtn->setText( ( "OK" ) );

	button2FrmLayout->addWidget( okSetBtn, 0, 1 );
	QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	button2FrmLayout->addItem( spacer_4, 0, 0 );

	canselSetBtn = new QPushButton( button2Frm, "canselSetBtn" );
	canselSetBtn->setText( ( "Cansel" ) );

	button2FrmLayout->addWidget( canselSetBtn, 0, 3 );

	tabID2Layout->addWidget( button2Frm, 1, 0 );

	baseSWd = new SettingBase( tabID2, "baseSWd" );

	tabID2Layout->addWidget( baseSWd, 0, 0 );

	selectTab->insertTab( tabID2, ( "SETTING" ) );

	/* メイン初期化 */
	QPixmap	timeIconPxm( ( const char** ) timeIcon_data );
	QPixmap	settingIconPxm( ( const char** ) settingIcon_data );

	selectTab->changeTab( tabID1, QIconSet(timeIconPxm), QString::fromUtf8( "時刻" ) );
	datetimeLbl->setText( QString::fromUtf8( "System Time" ) );
	blacket1Lbl->setText( "(" );
	blacket2Lbl->setText( ")" );
	ntpGBox->setTitle( QString::fromUtf8( "SNTP" ) );
	hostnameLbl->setText( QString::fromUtf8( "Host" ) );

	selectTab->changeTab( tabID2, QIconSet(settingIconPxm), QString::fromUtf8( "設定" ) );

	baseSWd->vItem->hostListGBox->setTitle( QString::fromUtf8( "NTPｻｰﾊﾞ名/IPｱﾄﾞﾚｽ" ) );
	baseSWd->vItem->defaultBtn->setText( QString::fromUtf8( "初期設定に戻す" ) );
	baseSWd->vItem->timeoutLbl1->setText( QString::fromUtf8( "タイムアウト" ) );
	baseSWd->vItem->timeoutLbl2->setText( QString::fromUtf8( "msec" ) );

#ifdef DTK_APPLET
	onlineBtn->setText( QString::fromUtf8( "　実行　" ) );

	baseSWd->vItem->syncPppChk->setText( QString::fromUtf8( "ネットワーク接続と連動する" ) );
	baseSWd->vItem->autoQuitChk->setText( QString::fromUtf8( "時刻を修正後、ｱﾌﾟﾚｯﾄを閉じる" ) );

	baseSWd->vItem->largeFontViewChk->setText( QString::fromUtf8( "大きいフォントで表示する" ) );
#else
	onlineBtn->setText( QString::fromUtf8( "実行" ) );
	quitBtn->setText( QString::fromUtf8( "終了" ) );

	baseSWd->vItem->autoQuitChk->setText( QString::fromUtf8( "時刻を修正後、終了する" ) );
	baseSWd->vItem->pppStopChk->setText( QString::fromUtf8( "終了時にﾈｯﾄﾜｰｸ接続を切断する" ) );
	baseSWd->vItem->pppStopChk2->setText( QString::fromUtf8( "ﾈｯﾄﾜｰｸ接続中に起動した場合は切断" ) );
#endif

	baseSWd->vItem->notfy2AdjustChk->setText( QString::fromUtf8( "実行時にﾈｯﾄﾜｰｸ接続を確認する" ) );
	baseSWd->vItem->autoStartChk->setText( QString::fromUtf8( "起動時に時刻修正を実行する" ) );

	okSetBtn->setText( QString::fromUtf8( " OK " ) );
	canselSetBtn->setText( QString::fromUtf8( "取消" ) );

	messageViewMed->setReadOnly( true );

	connect( selectTab, SIGNAL(currentChanged(QWidget *)), this, SLOT(pressTabSlot(QWidget *)) );
	connect( onlineBtn, SIGNAL(clicked()), this, SLOT(pressOnlineBtnSlot()) );
	connect( baseSWd->vItem->defaultBtn, SIGNAL(clicked()), this, SLOT(pressDefaultBtnSlot()) );
	connect( okSetBtn, SIGNAL(clicked()), this, SLOT(pressOkSetBtnSlot()) );
	connect( canselSetBtn, SIGNAL(clicked()), this, SLOT(pressCanselSetBtnSlot()) );

#ifdef DTK_APPLET
	connect( baseSWd->vItem->syncPppChk, SIGNAL(clicked()), this, SLOT(pressSyncPppChkSlot()) );
#else
	connect( quitBtn, SIGNAL(clicked()), this, SLOT(pressQuitBtnSlot()) );

	connect( baseSWd->vItem->pppStopChk, SIGNAL(clicked()), this, SLOT(pressPppStopChkSlot()) );
	connect( baseSWd->vItem->pppStopChk2, SIGNAL(clicked()), this, SLOT(pressPppStopChk2Slot()) );
#endif

	S_FontSize[0]			= this->font().pointSize();
	S_FontSize[1]			= (int)( ((double)S_FontSize[0] / (double)3) * (double)2 );

#ifdef DTK_APPLET
	S_LargeFontDisp			= false;
#else
	S_LargeFontDisp			= true;
#endif

	S_SyncPppUp				= false;
	S_PppDisconnected		= true;
	S_EnableTimeAdjustExec	= true;

	// 機種判別
	QString		qstZProd = SlDeviceInfo::getProductName();

	if( (qstZProd == "SL-A300") || (qstZProd == "") )
	{
		S_SL300Flag = true;
	}
	else
	{
		S_SL300Flag = false;
	}

	aplVarList			= new AplVarList( this );
	hqPppEnv 			= new HQPppEnv("dtktime","PPP",this);
	qtrDisconnectCnt	= new QTimer( this );

	connect( hqPppEnv, SIGNAL( netConnectingSignal()), this, SLOT( recvNetConnectSlot()) );
	connect( qtrDisconnectCnt, SIGNAL( timeout()), this, SLOT( recvNetDisconnectSlot()) );
}

MainFm::~MainFm()
{
	qtrDisconnectCnt->stop();

	disconnect( qtrDisconnectCnt );
	disconnect( hqPppEnv );

	delete qtrDisconnectCnt;
	delete hqPppEnv;
	delete aplVarList;
}

//event
bool MainFm::event( QEvent* ev )
{
	bool	blResult = QWidget::event( ev );

	if( ev->type() == QEvent::ApplicationFontChange )
	{
		setDispFontSize();
	}

	return blResult;
}

void MainFm::showEvent(QShowEvent* evShow)
{
	setDispFontSize();

	messageViewMed->clear();
	dispTimeDigitSlot();

	aplVarList->loadVarList();	// 設定ロード

#ifdef DTK_APPLET
	if( S_SyncPppUp == true )
	{
		// "ネットワーク接続と連動する" がＯＮの場合、自動実行は必ずＯＮ
		aplVarList->blAutoStart = true;
	}
#endif

	qtrClockInt	= new QTimer( this );
	blIsSetupTabReady	= false;

	connect( qtrClockInt, SIGNAL(timeout()), this, SLOT(dispTimeDigitSlot()) );

	setHostListCBoxItem();	// ntpホストリストを作成
	qtrClockInt->start( 1000, FALSE );

	if( S_SL300Flag == true )
	{
		// SL-A300か qvfbの場合、PPP接続確認/切断はしない
		aplVarList->blUpNotfy2Adjust	= false;

		aplVarList->blPPPstop			= false;
		aplVarList->blPPPstop2			= false;
	}

	// PPP接続の状態を調べる
	// result = 1: ppp / = 2: ether / 0:not connect / -1:error
	if(hqPppEnv->IsConnecting() > 0) aplVarList->blIsAlreadyConnection = true;

	// 通常始動
	if(aplVarList->blUpNotfy2Adjust == true)
	{
		if(aplVarList->blAutoStart == true)
		{
			// 起動時に時刻修正を実行.. が ONの場合の自動実行処理
			pressOnlineBtnSlot();
		}
		else
		{
			blPppUp2AdjustTrigger = false;
		}
	}
	else
	{
		blPppUp2AdjustTrigger = false;

		if(aplVarList->blAutoStart == true)
		{
			// 起動時に時刻修正を実行.. が ONの場合の自動実行処理
			QTimer::singleShot( 500, this, SLOT(execTimeAdjustSlot()) );
		}
	}

	QWidget::showEvent(evShow);
}

void MainFm::hideEvent(QHideEvent* evHide)
{
#ifdef DTK_APPLET
	if(selectTab->currentPage() == tabID2) pressCanselSetBtnSlot();
#endif

	qtrClockInt->stop();
	disconnect(qtrClockInt);

	if(aplVarList->blSetSystemClock == true) aplVarList->writeTrigger();	// トリガ書出し

	aplVarList->iSelectedHost = hostListCBox->currentItem();
	aplVarList->saveVarList();

	// 「ﾈｯﾄﾜｰｸ接続中に起動した場合は切断」がONで起動時にPPP接続がされていた場合か、
	// 「終了時にﾈｯﾄﾜｰｸ接続を切断する」が ONの場合
	if( ((aplVarList->blPPPstop2 == true) && (aplVarList->blIsAlreadyConnection == true)) || (aplVarList->blPPPstop == true) )
	{
		hqPppEnv->disconnectPPP();
	}

	delete qtrClockInt;

	QWidget::hideEvent(evHide);
}

void MainFm::keyPressEvent( QKeyEvent *e )
{
	QWidget*		qwdPage;

	qwdPage = selectTab->currentPage();

	switch( e->key() )
	{
		case Key_F33:
		case Key_Return:
			if(qwdPage == tabID1)
			{
				pressOnlineBtnSlot();
			}
			else if(qwdPage == tabID2)
			{
				pressOkSetBtnSlot();
			}
			break;

		case Key_Escape:
			if(qwdPage == tabID1)
			{
				this->close();
			}
			else if(qwdPage == tabID2)
			{
				pressCanselSetBtnSlot();
			}
			break;
	}
}

//public
#ifdef DTK_APPLET

QSize MainFm::sizeHint() const
{
	QSize		qszResult;

	if(isVgaDisp() == true)
	{
		/* Is VGA */
		if(S_LargeFontDisp == true)
		{
			qszResult.setWidth(WDG_VGA_FixL_WIDTH);
			qszResult.setHeight(WDG_VGA_FixL_HEIGHT);
		}
		else
		{
			qszResult.setWidth(WDG_VGA_FixS_WIDTH);
			qszResult.setHeight(WDG_VGA_FixS_HEIGHT);
		}
	}
	else
	{
		/* Is QVGA */
		qszResult.setWidth(WDG_QVGA_FixWIDTH);
		qszResult.setHeight(WDG_QVGA_FixHEIGHT);
	}

	if( qszResult.width() == 0 ) qszResult.setWidth( maximumWidth() );
	if( qszResult.height() == 0 ) qszResult.setHeight( maximumHeight() );

    return qszResult;
}

#endif

bool MainFm::isVgaDisp() const
{
	bool	blResult;

	if (qApp->desktop()->width() > 250)
	{
		blResult = true;
	}
	else
	{
		blResult = false;
	}

	return blResult;
}

//private slots
void MainFm::disposeSocketSlot()
{
	disconnect(hqSntpfx);

	delete hqSntpfx;

	S_EnableTimeAdjustExec = true;
}

void MainFm::recvSetSystemClockSlot()
{
	// SystemClock 変更シグナルを受け取った
	aplVarList->blSetSystemClock = true;
}

void MainFm::recvAutoQuitRequestSlot()
{
#ifdef DTK_APPLET
	QString		qstMsg(QString::fromUtf8("終了します..."));
#else
	QString		qstMsg(QString::fromUtf8("ｱﾌﾟﾘｹｰｼｮﾝを終了します..."));
#endif

	displayMessageStrSlot(qstMsg);

	qtrAutoQuitTimer = new QTimer( this );

	connect( qtrAutoQuitTimer, SIGNAL(timeout()), this, SLOT(autoQuitSlot()) );

	qtrAutoQuitTimer->start( 2000,TRUE );
}

void MainFm::autoQuitSlot()
{
	delete(qtrAutoQuitTimer);

	if(aplVarList->blAppAutoQuit == true)
	{
		pressQuitBtnSlot();
	}
}

void MainFm::recvNetConnectSlot()
{
	if(isVisible() == false)
	{
		blPppUp2AdjustTrigger = false;

		if( S_PppDisconnected == true )
		{
			if( S_SyncPppUp == true )
			{
				raise();
				show();
			}
		}
	}
	else
	{
		if(blPppUp2AdjustTrigger == true)
		{
			blPppUp2AdjustTrigger = false;

			execTimeAdjustSlot();
		}
	}

	S_PppDisconnected = false;

	/* 15秒間 up() を受けない場合、PPPが切断された物とみなす */
	qtrDisconnectCnt->start( 15000 );
}

void MainFm::recvNetDisconnectSlot()
{
	qtrDisconnectCnt->stop();

	S_PppDisconnected = true;
}

void MainFm::execTimeAdjustSlot()
{
	if(S_EnableTimeAdjustExec == true)
	{
		/* ２重実行を防ぐ */
		S_EnableTimeAdjustExec = false;

		QString		qstStrTmp;

		messageViewMed->clear();

		qstStrTmp = commentOutStr(hostListCBox->currentText());

		hqSntpfx = new HQSntp(qstStrTmp,123,aplVarList->iTimeoutMsec,aplVarList->blAppAutoQuit,this);

		connect( hqSntpfx, SIGNAL(requestDisposeHQSntpSignal()), this, SLOT(disposeSocketSlot()) );
		connect( hqSntpfx, SIGNAL(requestDispMessageHQSntpSignal(QString &)), this, SLOT(displayMessageStrSlot(QString &)) );
		connect( hqSntpfx, SIGNAL(setSystemClockSignal()), this, SLOT(recvSetSystemClockSlot()) );
		connect( hqSntpfx, SIGNAL(requestAutoQuitRequestSignal()), this, SLOT(recvAutoQuitRequestSlot()) );

		hqSntpfx->sendNtpPacket();
	}
}

void MainFm::displayMessageStrSlot(QString &qstDispStr)
{
	messageViewMed->insertLine( qstDispStr );
	messageViewMed->setCursorPosition ( messageViewMed->numLines() - 1, 0, FALSE );
}

void MainFm::pressTabSlot(QWidget *qwdTab)
{
	if(qwdTab == tabID1)
	{
		if(blIsSetupTabReady == true)
		{
			int			iSelectedHostId;
			int			iTimeout;
			bool		blNotfyAdj;
			bool		blStop;
			bool		blStop2;
			bool		blStart;
			bool		blAutoQuit;
			QString		qstTmp;

			getSetupTabDatas(qstTmp,iTimeout,blNotfyAdj,blStop,blStop2,blStart,blAutoQuit);

			iSelectedHostId = hostListCBox->currentItem();

			aplVarList->setVarList(qstTmp,iSelectedHostId,iTimeout,blNotfyAdj,blStop,blStop2,blStart,blAutoQuit);

			setHostListCBoxItem();

			blIsSetupTabReady = false;
		}
	}
	else if(qwdTab == tabID2)
	{
		bool	blAutoQuit = aplVarList->blAppAutoQuit;

		aplVarList->blAppAutoQuit = false;

		setSetupTabDatas(blAutoQuit);

		blIsSetupTabReady = true;
	}
}

void MainFm::pressOnlineBtnSlot()
{
	if(aplVarList->blUpNotfy2Adjust == true)
	{
		// PPP接続実行と、up() メッセージの受信で時刻修正を開始
		blPppUp2AdjustTrigger = true;
		hqPppEnv->connectPPP();
	}
	else
	{
		// 直接時刻修正を開始
		blPppUp2AdjustTrigger = false;
		execTimeAdjustSlot();
	}
}

void MainFm::pressDefaultBtnSlot()
{
#ifdef DTK_APPLET

	QString		qstTmp;

	aplVarList->getDefaultHostList(qstTmp);

	baseSWd->vItem->hostListMEdt->setText(qstTmp);

#else

	QString		qstStr1(QString::fromUtf8("どんとこいタイム"));
	QString		qstStr2(QString::fromUtf8("NTPサーバリストを初期設定に戻します\n\nよろしいですか？"));

	if( QMessageBox::information(this, qstStr1,  qstStr2, QMessageBox::Ok, QMessageBox::Cancel) ==  QMessageBox::Ok )
	{
		QString		qstTmp;

		aplVarList->getDefaultHostList(qstTmp);

		baseSWd->vItem->hostListMEdt->setText(qstTmp);
	}

#endif
}

void MainFm::pressSyncPppChkSlot()
{
#ifdef DTK_APPLET
	if( baseSWd->vItem->syncPppChk->isChecked() == true )
	{
		baseSWd->vItem->autoStartChk->setChecked(true);
		baseSWd->vItem->autoQuitChk->setChecked(true);
		baseSWd->vItem->notfy2AdjustChk->setChecked(false);

		baseSWd->vItem->autoStartChk->setEnabled(false);
	}
	else
	{
		baseSWd->vItem->notfy2AdjustChk->setChecked(true);

		baseSWd->vItem->autoStartChk->setEnabled(true);
	}
#endif
}

void MainFm::pressPppStopChkSlot()
{
#ifndef DTK_APPLET
	if(baseSWd->vItem->pppStopChk->isChecked() == true) baseSWd->vItem->pppStopChk2->setChecked(false);
#endif
}

void MainFm::pressPppStopChk2Slot()
{
#ifndef DTK_APPLET
	if(baseSWd->vItem->pppStopChk2->isChecked() == true) baseSWd->vItem->pppStopChk->setChecked(false);
#endif
}

void MainFm::pressOkSetBtnSlot()
{
	blIsSetupTabReady = true;

	selectTab->showPage(tabID1);

#ifdef DTK_APPLET
	this->close();
#endif
}

void MainFm::pressCanselSetBtnSlot()
{
	blIsSetupTabReady = false;

	selectTab->showPage(tabID1);

#ifdef DTK_APPLET
	this->close();
#endif
}

void MainFm::pressQuitBtnSlot()
{
	this->close();
}

void MainFm::dispTimeDigitSlot()
{
	QTime		qtTime = QTime::currentTime();
	QDate		qtDate = QDate::currentDate();

	QString		qstDispTimeStr;
	QString		qstDispDateStr;
	QString		qstDispDayname;

	qstDispTimeStr.sprintf("%.2d:%.2d:%.2d",qtTime.hour(),qtTime.minute(),qtTime.second());
	qstDispDateStr.sprintf("%4d.%.2d.%.2d",qtDate.year(),qtDate.month(),qtDate.day());

	switch(qtDate.dayOfWeek())
	{
		case 1:
			qstDispDayname = QString::fromUtf8("月");
			break;
		case 2:
			qstDispDayname = QString::fromUtf8("火");
			break;
		case 3:
			qstDispDayname = QString::fromUtf8("水");
			break;
		case 4:
			qstDispDayname = QString::fromUtf8("木");
			break;
		case 5:
			qstDispDayname = QString::fromUtf8("金");
			break;
		case 6:
			qstDispDayname = QString::fromUtf8("土");
			break;
		case 7:
			qstDispDayname = QString::fromUtf8("日");
			break;
	}

	timeLbl->setText(qstDispTimeStr);
	dateLbl->setText(qstDispDateStr);
	dayLbl->setText(qstDispDayname);
}

//private
void MainFm::setDispFontSize()
{
	if(isVgaDisp() == true)
	{
		int		iFontS;

		if(S_LargeFontDisp == true)
		{
			iFontS = 0;
		}
		else
		{
			iFontS = 1;
		}

		QFont	qfDispFont( baseFrm->font() );

		qfDispFont.setPointSize( S_FontSize[iFontS] );

		baseFrm->setFont( qfDispFont );
	}
}

void MainFm::setHostListCBoxItem()
{
	QString		qstTmp;
	int			iCnt;
	int			iTmp;

	hostListCBox->clear();

	for(iCnt = 0;iCnt < aplVarList->iHostListCnt;iCnt++)
	{
		iTmp = aplVarList->getHostName(iCnt,qstTmp);
		if(iTmp == -1)
		{
			break;
		}
		else if(iTmp > 0)
		{
			hostListCBox->insertItem(qstTmp);
		}
	}

	if(aplVarList->iSelectedHost < hostListCBox->count())
	{
		hostListCBox->setCurrentItem(aplVarList->iSelectedHost);
	}
}

void MainFm::setSetupTabDatas(bool blAutoQuit)
{
	baseSWd->vItem->hostListMEdt->setText(aplVarList->qstHostListStr);
	baseSWd->vItem->timeOutSbx->setValue(aplVarList->iTimeoutMsec);

#ifdef DTK_APPLET
	baseSWd->vItem->syncPppChk->setChecked(S_SyncPppUp);
	if(S_SyncPppUp == true)
	{
		baseSWd->vItem->autoStartChk->setChecked(true);

		baseSWd->vItem->autoStartChk->setEnabled(false);
	}
	else
	{
		baseSWd->vItem->autoStartChk->setEnabled(true);
	}

	baseSWd->vItem->largeFontViewChk->setChecked(S_LargeFontDisp);

	if(isVgaDisp() == false) baseSWd->vItem->largeFontViewChk->setEnabled(false);

#else
	baseSWd->vItem->pppStopChk->setChecked(aplVarList->blPPPstop);
	baseSWd->vItem->pppStopChk2->setChecked(aplVarList->blPPPstop2);

	if(S_SL300Flag == true) baseSWd->vItem->pppStopChk->setEnabled( false );
	if(S_SL300Flag == true) baseSWd->vItem->pppStopChk2->setEnabled( false );
#endif

	baseSWd->vItem->notfy2AdjustChk->setChecked(aplVarList->blUpNotfy2Adjust);
	if(S_SL300Flag == true) baseSWd->vItem->notfy2AdjustChk->setEnabled( false );

	baseSWd->vItem->autoStartChk->setChecked(aplVarList->blAutoStart);
	baseSWd->vItem->autoQuitChk->setChecked(blAutoQuit);
}

void MainFm::getSetupTabDatas(QString &qstHostList,int &iTimeout,bool &blNotfy2Adj, bool &blStop,bool &blStop2,bool &blStart,bool &blAutoQuit)
{
	qstHostList		= baseSWd->vItem->hostListMEdt->text();
	iTimeout		= baseSWd->vItem->timeOutSbx->value();

#ifdef DTK_APPLET
	S_SyncPppUp		= baseSWd->vItem->syncPppChk->isChecked();
	S_LargeFontDisp	= baseSWd->vItem->largeFontViewChk->isChecked();

	blStop			= false;
	blStop2			= false;
#else
	blStop			= baseSWd->vItem->pppStopChk->isChecked();
	blStop2			= baseSWd->vItem->pppStopChk2->isChecked();
#endif

	blNotfy2Adj		= baseSWd->vItem->notfy2AdjustChk->isChecked();
	blStart			= baseSWd->vItem->autoStartChk->isChecked();
	blAutoQuit		= baseSWd->vItem->autoQuitChk->isChecked();
}

QString MainFm::commentOutStr(QString qstStr)
{
	int		iTmp;

	iTmp = qstStr.find("(");

	if(iTmp >= 0) qstStr = qstStr.left(iTmp);

	qstStr = qstStr.stripWhiteSpace();

	return qstStr;
}
